@@ -43,7 +43,8 @@ linc_close_fd (LINCConnection *cnx)
 {
 	if (cnx->priv->fd >= 0) {
 		d_printf ("Close %d\n", cnx->priv->fd);
-		close (cnx->priv->fd);
+
+		LINC_CLOSE (cnx->priv->fd);
 	}
 	cnx->priv->fd = -1;
 }
@@ -339,6 +340,13 @@ linc_connection_initiate (LINCConnection        *cnx,
 	saddr = linc_protocol_get_sockaddr (
 		proto, host, service, &saddr_len);
 
+	if (!saddr && (strcmp (proto_name, "IPv6") ==0)) {/* Falling back to IPv4 */
+		proto = linc_protocol_find ("IPv4");
+		
+		saddr = linc_protocol_get_sockaddr (
+			proto, host, service, &saddr_len);
+	}
+
 	if (!saddr)
 		return FALSE;
 
@@ -371,7 +379,7 @@ linc_connection_initiate (LINCConnection        *cnx,
  out:
 	if (!retval && fd >= 0) {
 		d_printf ("initiation failed\n");
-		close (fd);
+		LINC_CLOSE (fd);
 	}
 
 	g_free (saddr);
@@ -435,6 +443,8 @@ linc_connection_read (LINCConnection *cnx,
 #endif
 			n = read (cnx->priv->fd, buf, len);
 
+		g_assert (n <= len);
+
 		if (n < 0) {
 #ifdef LINC_SSL_SUPPORT
 			if (cnx->options & LINC_CONNECTION_SSL) {
@@ -468,7 +478,6 @@ linc_connection_read (LINCConnection *cnx,
 
 		} else if (n == 0) {
 			d_printf ("we got EOF on fd %d\n", cnx->priv->fd);
-			linc_connection_state_changed (cnx, LINC_DISCONNECTED);
 			return LINC_IO_FATAL_ERROR;
 		} else {
 			buf += n;
@@ -484,14 +493,19 @@ linc_connection_read (LINCConnection *cnx,
 
 /* Determine the maximum size of the iovec vector */
 
-#if defined (UIO_MAXIOV) /* Glibc */
-# define LINC_IOV_MAX (UIO_MAXIOV)
-#elif defined (MAXIOV) /* HPUX */
+#if defined (MAXIOV) /* HPUX */
 # define LINC_IOV_MAX (MAXIOV)
 #elif defined (IOV_MAX) /* AIX */
 # define LINC_IOV_MAX (IOV_MAX)
 #elif defined (_SC_IOV_MAX) /* SGI */
 # define LINC_IOV_MAX_INIT (sysconf (_SC_IOV_MAX))
+#elif defined (__APPLE__)
+/* Even though the write(2) man page mentions it, UIO_MAXIOV is only
+ * available if KERNEL is defined on MacOS X 10.1
+ */
+#  define LINC_IOV_MAX 1024
+#elif defined (UIO_MAXIOV) /* Glibc */
+# define LINC_IOV_MAX (UIO_MAXIOV)
 #else /* Safe Guess */
 # define LINC_IOV_MAX 16
 #endif
@@ -600,6 +614,19 @@ write_data (LINCConnection *cnx, QueuedWrite *qw)
 	return bytes_written;
 }
 
+static gboolean
+linc_connection_should_block (LINCConnection      *cnx,
+			      const LINCWriteOpts *opt_write_opts)
+{
+	if (!opt_write_opts)
+		return TRUE;
+
+	if (opt_write_opts->block_on_write)
+		return TRUE;
+
+	return FALSE;
+}
+
 /**
  * linc_connection_writev:
  * @cnx: the connection to write to
@@ -647,13 +674,17 @@ linc_connection_writev (LINCConnection       *cnx,
 	status = write_data (cnx, &qw);
 
 	if (status == LINC_IO_QUEUED_DATA) {
+		if (linc_get_threaded ())
+			g_error ("Big foo here");
+
 		/* Queue data & listen for buffer space */
 		linc_watch_set_condition (cnx->priv->tag,
 					  LINC_ERR_CONDS | LINC_IN_CONDS | G_IO_OUT);
-		if (opt_write_opts &&
-		    !opt_write_opts->block_on_write) {
+
+		if (!linc_connection_should_block (cnx, opt_write_opts)) {
 			queue_flattened (cnx, qw.vecs, qw.nvecs);
 			return LINC_IO_QUEUED_DATA;
+
 		} else {
 			linc_main_iteration (TRUE);
 			goto continue_write;
